package com.example.takeawaysystemserver.service.impl;

import com.auth0.jwt.interfaces.Claim;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.example.takeawaysystemserver.constant.CommonConstant;
import com.example.takeawaysystemserver.entity.User;
import com.example.takeawaysystemserver.model.dto.UserLoginDTO;
import com.example.takeawaysystemserver.model.dto.UserPasswordDTO;
import com.example.takeawaysystemserver.model.dto.UserRegisterDTO;
import com.example.takeawaysystemserver.model.dto.UserUpdateDTO;
import com.example.takeawaysystemserver.service.RedisService;
import com.example.takeawaysystemserver.service.UserService;
import com.example.takeawaysystemserver.mapper.UserMapper;
import com.example.takeawaysystemserver.util.DESUtil;
import com.example.takeawaysystemserver.util.JwtUtil;
import com.example.takeawaysystemserver.util.R;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
* @author qiushengyu
* @description 针对表【user】的数据库操作Service实现
* @createDate 2024-07-11 17:14:00
*/
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User>
    implements UserService{

    private final DESUtil desUtil;

    private final RedisService redisService;

    public UserServiceImpl(DESUtil desUtil, RedisService redisService) {
        this.desUtil = desUtil;
        this.redisService = redisService;
    }

    @Override
    public R<String> register(UserRegisterDTO userRegisterDTO) {
        if (checkUser(userRegisterDTO)){
            return R.fail("用户已被注册");
        }
        String password = desUtil.SHA512(userRegisterDTO.getPassword());
        User user = User.builder()
                .username(userRegisterDTO.getUsername())
                .password(password)
                .phone(userRegisterDTO.getPhone())
                .status(1)
                .build();
        int insert = baseMapper.insert(user);
        if (insert == 1){
            return R.ok("注册成功");
        }
        return R.ok("注册失败");
    }

    @Override
    public R<String> login(UserLoginDTO userLoginDTO) {
        User user = baseMapper.selectOne(new LambdaQueryWrapper<User>().eq(User::getUsername, userLoginDTO.getUsername()));
        if (Objects.isNull(user)){
            return R.fail("未找到用户，请注册");
        }
        String password = desUtil.SHA512(userLoginDTO.getPassword());
        if (!password.equals(user.getPassword())){
            return R.fail("密码错误，请重试");
        }
        Map<String, String> map = new HashMap<>();
        map.put("id", user.getId().toString());
        String token = JwtUtil.getToken(map, CommonConstant.TOKEN_EXPIRE_TIME);
        redisService.hSet(CommonConstant.LOGIN_USER, user.getId().toString(), user, CommonConstant.EXPIRE_TIME);
        return R.ok(token);
    }

    @Override
    public R<String> logout(String token) {
        redisService.hDel(CommonConstant.LOGIN_USER,getUserId(token));
        return R.ok("退出成功");
    }

    @Override
    public R<String> updateUserInfo(String token, UserUpdateDTO userUpdateDTO) {
        Integer id = Integer.parseInt(getUserId(token));
        LambdaUpdateWrapper<User> wrapper = new LambdaUpdateWrapper<User>()
                .eq(User::getId, id)
                .set(User::getUsername,userUpdateDTO.getUsername())
                .set(User::getPhone, userUpdateDTO.getPhone());
        if (baseMapper.update(wrapper) == 1){
            return R.ok("修改成功");
        }
        return R.fail("修改失败");
    }

    @Override
    public R<String> updatePassword(String token, UserPasswordDTO userPasswordDTO) {
        if (!userPasswordDTO.getNewPassword().equals(userPasswordDTO.getConfirm())){
            return R.fail("两次密码不一致，请重试");
        }
        String password = desUtil.SHA512(userPasswordDTO.getNewPassword());
        Integer id = Integer.parseInt(getUserId(token));
        LambdaUpdateWrapper<User> wrapper = new LambdaUpdateWrapper<User>()
                .eq(User::getId, id)
                .set(User::getPassword,password);
        if (baseMapper.update(wrapper)  == 1){
            return R.ok("修改成功");
        }
        return R.fail("修改失败");
    }

    private Boolean checkUser(UserRegisterDTO userRegisterDTO){
        User user = baseMapper.selectOne(new LambdaQueryWrapper<User>()
                .select(User::getUsername)
                .eq(User::getUsername, userRegisterDTO.getUsername()));
        return Objects.nonNull(user);
    }

    public String getUserId(String token){
        Map<String, Claim> map = JwtUtil.getPayload(token);
        return map.get("id").toString().replaceAll("\"", "");
    }
}




